import os
import sys
import datetime
import pygame
import pygame.gfxdraw
from .maths import *

UNIT = 0
TEXUNIT = 32
width = 0
height = 0
window = None
lighting = None
shouldclose = False
keys = {}
keys_times = {}
delta = 0.0
timer = 0.0
frameid = -1
fullscreen = False
camera = None

fpssurf = None
lastfps = -1

debugfont = None
lighting = None

FPS = 60

clock = pygame.time.Clock()

def initialize():
	global debugfont
	pygame.mixer.init(44100, -16, 2, 1024)
	pygame.init()
	debugfont = pygame.font.SysFont('Arial', 15)
	print('SOT: initialized')
	print('SOT: used driver:',pygame.display.get_driver())

class Camera:
	def __init__(self, pos):
		self.pos = pos

	def unproject(self, x, y):
		x -= width/2
		y -= height/2
		x /= UNIT
		y /= UNIT
		x += camera.pos.x
		y -= camera.pos.y
		y = -y
		return (x, y)

	def apply(self, x, y):
		assert(width)
		assert(height)

		x -= self.pos.x
		y -= self.pos.y

		y = -y

		x *= UNIT
		y *= UNIT

		x += width/2
		y += height/2

		return int(x), int(y)

def initialize_display(_width, _height, _fullscreen=False):
	global UNIT, width, height, window, fullscreen, lighting

	width = _width
	height = _height
	set_fullscreen(_fullscreen)

	width, height = window.get_size()
	UNIT = height//12

	pygame.display.set_caption("Souls Of The Tower [PyWeek Demo]")

	lighting = pygame.Surface((width, height), pygame.HWSURFACE)

def set_fullscreen(flag):
	global fullscreen, window
	fullscreen = flag
	if flag:
		window = pygame.display.set_mode((width, height), pygame.FULLSCREEN|pygame.HWSURFACE|pygame.SRCALPHA|pygame.DOUBLEBUF)
	else:
		window = pygame.display.set_mode((width, height), pygame.HWSURFACE|pygame.DOUBLEBUF|pygame.SRCALPHA)

def toogle_fullscreen():
	set_fullscreen(not fullscreen)

def set_camera(_camera):
	global camera
	camera = _camera

def flip(fps=FPS):
	pygame.display.flip()
	clock.tick(fps)

def is_pressed(key):
	return keys.get(key, False)

def is_tapped(key):
	return keys.get(key, False) and keys_times.get(key, -1) == frameid

def is_clicked(button):
	return keys.get('$%s'%button, False)

def is_just_clicked(button):
	return keys.get('$%s'%button, False) and keys_times.get('$%s'%button, -1) == frameid
	
def update_time():
	global delta, timer
	delta = clock.get_time()*0.001
	delta = min(0.1, delta)
	timer += delta

def cls(color):
	window.fill(color)

def draw(surface, x, y):
	x,y = camera.apply(x,y)
	width, height = surface.get_size()
	y -= height//2
	x -= width//2
	window.blit(surface, (x, y-1))

def draw_light(surface, x, y):
	x,y = camera.apply(x,y)
	width, height = surface.get_size()
	y -= height//2
	x -= width//2
	lighting.blit(surface, (x, y-1))

def draw_rect(x, y, width, height, color):
	x,y = camera.apply(x,y)
	width, height = width*UNIT, height*UNIT
	y -= height
	pygame.draw.rect(window, color, (x, y, int(width), int(height)))

def draw_pixel(x, y, color):
	x,y = camera.apply(x,y)
	pygame.gfxdraw.filled_circle(window, x,y, max(1, int(UNIT/32)), color)

def draw_polygon(polygon, color):
	points = []
	for point in polygon:
		x,y = point.x, point.y		
		x,y = camera.apply(x,y)
		points.append((x,y))
	pygame.gfxdraw.filled_polygon(window, points, color)

def draw_line(line, color):
	points = []
	for point in line:
		x,y = point.x, point.y
		x,y = camera.apply(x,y)
		points.append((x,y))
	pygame.draw.line(window, color, points[0], points[1])

def show_fps():
	global lastfps, fpssurf
	fps = clock.get_fps()
	if fpssurf is None or fps != lastfps:
		fpssurf = debugfont.render('fps %.1f' % fps, 1, (255,255,255))
	window.blit(fpssurf, (4,height-20))

def screenshot_path():
	now = datetime.datetime.now()
	if not os.path.isdir('screenshots'):
		os.makedir('screenshots')
	return now.strftime('screenshots/screenshot_%Y-%m-%d_%H:%M:%S.%f.png')

def save_screenshot():
	path = screenshot_path()
	print(path)
	pygame.image.save(window, path)

def events():
	global shouldclose, frameid
	frameid += 1
	for event in pygame.event.get():
		if event.type == pygame.QUIT:
			shouldclose = True
		if event.type == pygame.KEYDOWN:
			keys[event.key] = True
			keys_times[event.key] = frameid
		if event.type == pygame.KEYUP:
			keys[event.key] = False
			keys_times[event.key] = frameid

		if event.type == pygame.MOUSEBUTTONDOWN:
			keys['$%s'%event.button] = True
			keys_times['$%s'%event.button] = frameid

		if event.type == pygame.MOUSEBUTTONUP:
			keys['$%s'%event.button] = False
			keys_times['$%s'%event.button] = frameid

def finalize():
	pygame.quit()
	print('engine finalized')
