import math
import pygame
import random
import pygame.gfxdraw
from . import demo
from . import data
from . import engine
from . import keys
from . import sound
from . import parameters
from . import tiledmaps
from . import person as entities
from . import animation
from . import particles as ptcs
from .maths import *

font = None
small_font = None
dfont = None
attackfont = None
notefont = None
handfont = None
camera = engine.Camera(vec2(15,5))
player = None
tiledmap = None
backmap = None
persons = None
pointlight = None
vignette = None
bleedscreen = None
point_light = None
elevator_map = None
generator_room_map = None
personal_room_map = None
maps = {}
demo.floor = 0
generators = {}
particles = []

rand_sounds = 0

player_anims = {}
thing_anims = {}
knocker_anims = {}
panic_anims = {}
items_textures = {}

def initialize():
	engine.initialize()
	engine.initialize_display(*parameters.window_size, parameters.fullscreen)
	engine.set_camera(camera)
	sound.initialize()

def preload_resources():
	global tiledmap, backmap, point_light, vignette, bleedscreen
	global font, dfont, small_font, rand_sounds, note_img, notefont, handfont
	global elevator_map, generator_room_map, personal_room_map, attackfont

	pointlight = data.load_image('img/point_light.png')
	sound.load_sound("snd/rar.ogg", 'rar')
	sound.load_sound("snd/cabinet.ogg", 'cabinet')
	sound.load_sound("snd/paper.ogg", 'paper')
	sound.load_sound("snd/panic_end.ogg", 'panic_end')
	sound.load_sound("snd/electro.ogg", 'electro')
	sound.load_sound("snd/search.ogg", 'search')
	sound.load_sound("snd/beep.ogg", 'beep')
	sound.load_sound("snd/door.ogg", 'door')
	sound.load_sound("snd/click.ogg", 'click')
	sound.load_sound("snd/reload.ogg", 'reload')
	sound.load_sound("snd/screamer.ogg", 'screamer')
	sound.load_sound("snd/electric.ogg", 'electric')
	sound.load_sound("snd/elevator.ogg", 'elevator')
	sound.load_sound("snd/thing_breath.ogg", 'thing_breath')
	sound.load_sound("snd/door_knocking_left.ogg", 'door_knocking_left')
	sound.load_sound("snd/many_steps.ogg", 'many_steps')
	sound.load_sound("snd/many_steps_out.ogg", 'many_steps_out')
	sound.load_sound("snd/generator_start.ogg", 'generator_start')

	font = pygame.font.Font(data.load('THE_GONjURING.ttf'), 32)
	small_font = pygame.font.Font(data.load('THE_GONjURING.ttf'), 20)
	dfont = pygame.font.Font(data.load('Open_24_Display_St.ttf'),engine.UNIT//3)
	notefont = pygame.font.Font(data.load('mytype.ttf'),engine.UNIT//4)
	handfont = pygame.font.Font(data.load('ShadowsIntoLight-Regular.ttf'),engine.UNIT//4+4)
	attackfont = pygame.font.Font(data.load('ShadowsIntoLight-Regular.ttf'),engine.UNIT//2)

	for i in range(4):
		sound.load_sound("snd/step_%s.ogg" % i, 'step_%s' % i)
	for i in range(4):
		sound.load_sound("snd/nestep_%s.ogg" % i, 'nestep_%s' % i)
	for i in range(4):
		sound.load_sound("snd/smallroom_step_%s.ogg" % i, 'smallroom_step_%s' % i)

	rand_sounds = 5
	for i in range(rand_sounds):
		sound.load_sound("snd/rand_sound_%s.ogg" % i, 'rand_sound_%s' % i)

def load_resources():
	global tiledmap, backmap, point_light, vignette, bleedscreen
	global font, dfont, small_font, rand_sounds, note_img, notefont, handfont
	global elevator_map, generator_room_map, personal_room_map
	demo.load()

	personal_room_map = tiledmaps.load_map('personal_room.json')
	personal_room_map.build_image()

	elevator_map = tiledmaps.load_map('elevator.json')
	elevator_map.build_image()

	generator_room_map = tiledmaps.load_map('generator_room.json')
	generator_room_map.build_image()

	tiledmap = elevator_map

	backmap = tiledmaps.load_map('elevator_way.json')
	backmap.build_image()

	maps['elevator'] = elevator_map
	maps['personal_room'] = personal_room_map
	maps['generator_room'] = generator_room_map

	vignette = pygame.image.load(data.load('img/vignette.png'), "vignette.png")
	vignette = pygame.transform.scale(vignette, (engine.width, engine.height)).convert_alpha()
	bleedscreen = pygame.image.load(data.load('img/bleed_screen.png'), "bleed_screen.png")
	bleedscreen = pygame.transform.scale(bleedscreen, (engine.width, engine.height)).convert_alpha()
	point_light = pygame.image.load(data.load('img/point_light.png'), "point_light.png")
	point_light = pygame.transform.scale(point_light, (engine.UNIT*8, engine.UNIT*8))
	items_textures['fuel'] = pygame.image.load(data.load('img/fuel.png'), 'fuel.png')
	items_textures['battery'] = pygame.image.load(data.load('img/battery.png'), 'battery.png')

	note_img = pygame.image.load(data.load('img/docs.png'), "docs.png")
	note_img = pygame.transform.scale(note_img, (engine.TEXUNIT*8, engine.TEXUNIT*16))

def setup_map():
	global player, persons, generators, particles, inventory
	particles = []
	generators = {}

	inventory = {'fuel':0, 'battery':0}

	walk_anim = animation.load_animation('img/walk.anim', (engine.UNIT, engine.UNIT*2), False)
	idle_anim = animation.load_animation('img/idle.anim', (engine.UNIT, engine.UNIT*2), False)
	walk_dark_anim = animation.load_animation('img/walk_dark.anim', (engine.UNIT, engine.UNIT*2), False)
	idle_dark_anim = animation.load_animation('img/idle_dark.anim', (engine.UNIT, engine.UNIT*2), False)
	interaction_anim = animation.load_animation('img/interaction.anim', (engine.UNIT, engine.UNIT*2), False)
	interaction_dark_anim = animation.load_animation('img/interaction_dark.anim', (engine.UNIT, engine.UNIT*2), False)
	player_anims['idle'] = idle_anim
	player_anims['walk'] = walk_anim
	player_anims['idle_dark'] = idle_dark_anim
	player_anims['walk_dark'] = walk_dark_anim
	player_anims['interaction'] = interaction_anim
	player_anims['interaction_dark'] = interaction_dark_anim


	run_anim = animation.load_animation('img/thing_run.anim', (engine.UNIT*4, engine.UNIT*2), True)
	idle_anim = animation.load_animation('img/thing_idle.anim', (engine.UNIT*4, engine.UNIT*2), True)
	thing_anims['idle'] = idle_anim
	thing_anims['walk'] = run_anim

	idle_anim = animation.load_animation('img/knocker_idle.anim', (engine.UNIT*4, engine.UNIT*2), False)
	knocker_anims['idle'] = idle_anim

	idle_anim = animation.load_animation('img/panic_idle.anim', (engine.UNIT, engine.UNIT*2), False)
	panic_anims['idle'] = idle_anim

	player = entities.Person(vec2(2,2), vec2(1,2), 2.5, player_anims, entities.AI_PLAYER)

	persons = [player,]

def spawn_directional_particles(count, origin, colors, velocity, life):
	for _ in range(count):
		vel = velocity.cpy()
		vel.x += (random.random()-0.5)*8
		vel.y += (random.random()-0.5)*8
		p = ptcs.Particle(origin.cpy(), vel, colors, max(0.1, life*random.random()))
		particles.append(p)

def ondead(screamer):
	if screamer:
		sound.play("screamer", True)

def setup_generator_room():
	if generators.get(demo.floor):
		generator_room_map.lighting = (200,70,0)
		generator_room_map.lighted = True
		sound.set_generator_noise(2)
	else:
		generator_room_map.lighting = (10,10,10)
		generator_room_map.lighted = False

def setup_personal_room():
	if generators.get(demo.floor):
		personal_room_map.lighting = (255,255,255)
		personal_room_map.lighted = True
		sound.set_generator_noise(1)
		#personal_room_map.lighting = (0,0,0)
		#personal_room_map.lighted = False
	else:
		personal_room_map.lighting = (0,0,0)
		personal_room_map.lighted = False

def main():
	print('Souls Of The Tower Copyright (C) 2019  MihailRis')
	initialize()
	preload_resources()
	menu()

SH = 5

def rehpic(text):
	coded = []
	for c in text:
		coded.append(chr(ord(c)-(SH+12)))
	return ''.join(coded)

LABEL1 = 'º1^zyrz}cz\x841CABJ' # :D i don't know what i doing

def menu():
	while not engine.shouldclose:
		engine.events()
		engine.update_time()
		pygame.mouse.set_visible(True)
		if engine.is_tapped(keys.K_RETURN):
			play()

		if engine.is_tapped(keys.K_ESCAPE):
			return

		engine.cls((0,0,0))
		engine.window.blit(font.render('Press Enter to play', 1, (255,255,255)), (100,100))
		engine.window.blit(font.render('Press Escape to quit', 1, (255,255,255)), (100,150))
		engine.window.blit(engine.debugfont.render('See README.txt', 1, (255,255,255)), (engine.width-150, 50))
		engine.window.blit(engine.debugfont.render('Twitter: @MihailRis', 1, (255,255,255)), (engine.width-170,20))
		engine.window.blit(engine.debugfont.render('Copyright '+rehpic(LABEL1), 1, (255,255,255)), (engine.width-200,engine.height-30))
		engine.show_fps()
		engine.flip()

def win():
	while not engine.shouldclose:
		engine.events()
		engine.update_time()
		sound.add_panic(-10)
		if engine.is_tapped(keys.K_q):
			return

		engine.cls((0,0,0))
		engine.window.blit(engine.debugfont.render('Thanks for playing! Development of the game will be continued.', 1, (255,255,255)), (100,100))
		engine.window.blit(engine.debugfont.render('You can follow the game with Discord server, see README.txt', 1, (255,255,255)), (100,116))
		engine.window.blit(font.render('Press Q to quit', 1, (255,255,255)), (100,200))
		engine.show_fps()
		engine.flip()

def play():
	global tiledmap
	load_resources()
	setup_map()

	flashlight = False
	lightworks = True
	backlight = 0.0
	flashingtimer = 0.0
	energy = 1.0

	deadtimer = 0.0
	elevator_timer = 0.0
	elevator_dir = 0
	elevator_speed = 2.0
	quake = 0.0
	fadetimer = 1.0
	interaction_timer = 0.0
	fadein = True
	random_sounds_timer = 0.0
	elevate_num = 0

	action = None
	drawerid = None
	mapname = None
	door_spawn_x = 0.0
	lighting_needs_update = True
	screamer = True

	battle = False
	battle_value = 0.0
	battle_timer = 0.0
	note_on = True

	#generators[10] = True
	#generators[20] = True
	#inventory['fuel'] = 1

	paused = False
	pygame.mouse.set_visible(False)
	
	while not engine.shouldclose:
		engine.events()
		engine.update_time()

		if engine.is_tapped(keys.K_ESCAPE):
			paused = not paused
			if paused:
				pygame.mouse.set_visible(True)
				pygame.mixer.pause()
			else:
				pygame.mouse.set_visible(False)
				pygame.mixer.unpause()
				if generators.get(demo.floor):
					if tiledmap == generator_room_map:
						setup_generator_room()
					elif tiledmap == personal_room_map:
						setup_personal_room()
					else:
						sound.set_generator_noise(0)
				else:
					sound.set_generator_noise(0)
		if not paused:
			pygame.mouse.set_pos(engine.width//2, engine.height//2)

		if paused and engine.is_tapped(keys.K_q):
			return

		if not paused:
			if quake > 0.0:
				quake -= engine.delta
				quake = max(0.0, quake)
			random_sounds_timer += engine.delta

			sound.set_battle(battle)
			sound.update(engine.delta, (not flashlight and not tiledmap.lighted))

			if deadtimer > 6:
				return

			if engine.is_tapped(keys.K_h):
				note_on = not note_on
				sound.play('paper')

			if battle:
				battle_timer -= engine.delta
				battle_value -= engine.delta*0.5
				battle_value = max(battle_value, 0.0)
				if engine.is_tapped(keys.K_SPACE):
					battle_value += 0.25*0.5
					if battle_value >= 1.0:
						battle = False
						fadein = True
						fadetimer = 1.0
						persons.clear()
						persons.append(player)
				elif battle_timer <= 0.0:
					player.dead = True
					battle = False
			elif sound.panic_timer > 1.3 and interaction_timer <= 0.0:
				interaction_timer = 2.0
				sound.add_panic(-10)
				action = 'panic_end'
				sound.play('rand_sound_0')

			if engine.is_tapped(keys.K_F11):
				engine.toogle_fullscreen()

			if engine.is_tapped(keys.K_F2):
				engine.save_screenshot()

			if interaction_timer <= 0.0:
				if (engine.is_tapped(keys.K_f) or engine.is_just_clicked(3)) and energy > 0.0 and not tiledmap.lighted:
					flashlight = not flashlight
					sound.play('click')
					if demo.floor in demo.knockers and tiledmap != elevator_map:
						battle = True
						battle_timer = 10.0
						battle_value = 0.0
						demo.knockers.remove(demo.floor)

						knocker = entities.Person(vec2(player.pos.x+player.dir*2,player.pos.y), vec2(1,2), 15.5, knocker_anims, entities.AI_KNOCKER)
						knocker.dir = -player.dir
						persons.append(knocker)

				if engine.is_tapped(keys.K_e) or engine.is_just_clicked(1):
					drawers = tiledmap.objects['drawers']
					for drawerid, drawerobj in enumerate(drawers):
						x = drawerobj[0]
						if player.pos.x >= x and player.pos.x <= x+1:
							interaction_timer = 1.5
							action = 'search'
							sound.play('search')
							break

					if tiledmap == generator_room_map and not generators.get(demo.floor):
						if inventory.get('fuel'):
							interaction_timer = 3.6+0.5
							action = 'start'
							sound.play('generator_start')
							player.dir = -1
							inventory['fuel'] -= 1

				if tiledmap == elevator_map and elevator_timer <= 0.0:
					if engine.is_tapped(keys.K_UP) or engine.is_tapped(keys.K_w):
						if demo.floor == demo.max_floor:
							sound.play('beep')
						else:
							elevator_dir = 1
							sound.play('beep')
							tiledmap.set(2,5, 'buttons', 25+1)
							elevator_timer = 1.0
							sound.play('elevator')
							quake += 0.1
							elevate_num += 1
							if random.random() > 0.98 or elevate_num in demo.elevator_sparkles:
								spawn_directional_particles(100, vec2(7,4), [(255,255,255), (255,255,0), (255,0,0), (0,0,0)], vec2(0,4), 1.0)
								sound.play('electro')
					elif engine.is_tapped(keys.K_DOWN) or engine.is_tapped(keys.K_s):
						if demo.floor == demo.min_floor:
							sound.play('beep')
						else:
							elevator_dir = -1 
							sound.play('beep')
							tiledmap.set(2,5, 'buttons', 25)
							elevator_timer = 1.0
							sound.play('elevator')
							quake += 0.2
				if elevator_timer > 0.0:
					elevator_timer -= engine.delta*0.1*elevator_speed
					if elevator_timer < 0.0:
						elevator_timer = 0.0
						quake += 0.3
						demo.floor += elevator_dir
						if demo.floor in demo.knockers:
							if random.random() > 0.6:
								sound.play('door_knocking_left')
								sound.add_panic(1.0)
							else:
								sound.add_panic(0.2)

				if not player.dead:
					if not tiledmap.lighted:
						if random_sounds_timer > 8.0 and random.random() > 0.999:
							sound.play('rand_sound_%s' % random.randrange(0, rand_sounds))
					if flashlight:
						energy -= engine.delta*0.01
					if flashlight and energy <= 0.0:
						flashingtimer = 1.0
						flashlight = False
					if flashingtimer > 0.0:
						flashingtimer -= engine.delta
						if random.random() > 0.9:
							lightworks = not lightworks
							if not lightworks:
								sound.play('electric')
						if flashingtimer <= 0.0:
							lightworks = True

					if not flashlight and energy <= 0.0 and flashingtimer <= 0.0 and inventory.get('battery'):
						action = 'reload'
						sound.play('reload')
						interaction_timer = 2.0

					left = engine.is_pressed(keys.K_LEFT) or engine.is_pressed(keys.K_a)
					right = engine.is_pressed(keys.K_RIGHT) or engine.is_pressed(keys.K_d)
					player.input['left'] = left
					player.input['right'] = right
					player.flashlight = flashlight

					for person in persons:
						person.update_AI(tiledmap, persons, engine.delta)
					for person in persons:
						person.controls(tiledmap, engine.delta)
						person.update_physics(tiledmap, engine.delta)

					doors = tiledmap.objects['doors']
					for door in doors:
						x,y,w,h, _,_ = door
						pos = player.pos
						if pos.x > x and pos.x < x+w and pos.y > y and pos.y < y+h:
							if elevator_timer <= 0.0 and interaction_timer <= 0.0:
								x,y,w,h, mapname, door_spawn_x = door
								fadetimer = 1.0
								interaction_timer = 1.0
								fadein = False
								action = 'door'
								player.input['left'] = False
								player.input['right'] = False
								sound.play('door')

				else:
					if deadtimer <= 0.0:
						battle = False
						ondead(screamer)
						flashlight = False
					deadtimer += engine.delta

			if interaction_timer > 0.0:
				interaction_timer -= engine.delta*0.5
				if interaction_timer <= 0.0:
					if action == 'reload':
						inventory['battery'] -= 1
						energy = 1.0
						flashlight = True
						sound.play('click')

					elif action == 'panic_end':
						screamer = False
						player.dead = True
						sound.play('panic_end')
						for i in range(30):
							x = random.random()*(tiledmap.width-2)+1
							person = entities.Person(vec2(x,2), vec2(1,2), 2.5, panic_anims, entities.AI_PLAYER)
							person.dir = 1 if player.pos.x > person.pos.x else -1
							persons.append(person)

					elif action == 'search':
						fullid = '%s_%s' % (demo.floor, drawerid)
						if fullid in demo.loot:
							name = demo.loot[fullid]
							inventory[name] += 1
							del demo.loot[fullid]

					elif action == 'start':
						generators[demo.floor] = True
						setup_generator_room()
						if flashlight:
							flashlight = False
							sound.play('click')
							backlight = 70

						if len(generators) >= 3:
							action = 'win'
							interaction_timer = 1.0
							fadein = False
							fadetimer = 1.0
							continue

					elif action == 'win':
						print("You won!")
						sound.set_generator_noise(0)
						win()
						return

					elif action == 'door':
						lighting_needs_update = True
						if mapname == 'mainmap':
							tiledmap = demo.room_maps[demo.roomids[demo.floor]]
						else:
							tiledmap = maps[mapname]
						if tiledmap.lighted:
							flashlight = False
							backlight = 70
							lighting_needs_update = True
						persons.clear()
						persons.append(player)
						if mapname == 'mainmap':
							if demo.floor % 10 == 0:
								mapname = 'personal_room'
								tiledmap = maps[mapname]
							else:
								if demo.floor in demo.things:
									spawn = tiledmap.objects['spawns'].get('thing')
									if spawn:
										thing = entities.Person(vec2(spawn[0],2), vec2(1,2), 15.5, thing_anims, entities.AI_THING1)
										thing.dir = 1
										persons.append(thing)
						if mapname == 'personal_room':
							setup_personal_room()
						elif mapname == 'generator_room':
							setup_generator_room()
						else:
							sound.set_generator_noise(0)
						tiledmap.build_image()
						#if mapname == 'elevator': todo
							#if random.randint(0,10) == 0:
								#persons.append()
						player.pos.x = tiledmap.objects['spawns'][door_spawn_x][0]
					interaction_timer = 0.0

			engine.cls((0,0,0))

			# -- lighting
			if player.dead:
				if engine.frameid % 2 == 0 or deadtimer > 0.8:
					engine.lighting.fill((255,255,255))
				else:
					engine.lighting.fill((0,255,255))
			elif battle:
				if (engine.frameid//4) % 2 == 0:
					engine.lighting.fill((0,255,255))
				else:
					engine.lighting.fill((255,255,255))
			else:
				light = [int(90//1.3+backlight),int(84//1.3+backlight),int(80//1.3+backlight)]
				light[0] -= tiledmap.lighting[0]
				light[1] -= tiledmap.lighting[1]
				light[2] -= tiledmap.lighting[2]
				light[0] = max(0, light[0])
				light[1] = max(0, light[1])
				light[2] = max(0, light[2])
				if light[0] or light[1] or light[2] or lighting_needs_update:
					lighting_needs_update = False
					engine.lighting.fill(light)
					cx = engine.width/2
					cy = engine.height/2
					if flashlight or (lightworks and flashingtimer > 0.0):
						if interaction_timer > 0.0:
							yoffset = int(interaction_timer > 0.0)*800
							pygame.gfxdraw.filled_polygon(engine.lighting, ((cx+player.dir*15,cy), (cx+1600*player.dir, cy+400+yoffset), (cx+1600*player.dir, cy-400+yoffset)), (35,40,45, 255))
						else:
							pygame.gfxdraw.filled_polygon(engine.lighting, ((cx,cy), (cx+1600*player.dir, cy+400), (cx+1600*player.dir, cy-400)), (35,40,45, 255))
						backlight = 10
					else:
						if backlight < 70:
							backlight += engine.delta*30
					if tiledmap == personal_room_map and generators.get(demo.floor):
						engine.draw_light(point_light, 12.5,2.5)
			# -- lighting end
			if tiledmap == elevator_map:
				engine.draw(backmap.surface, 4, (elevator_timer*backmap.height)*elevator_dir+backmap.height+2)
				engine.draw(backmap.surface, 4, (elevator_timer*backmap.height)*elevator_dir+2)
				engine.draw(backmap.surface, 4, (elevator_timer*backmap.height)*elevator_dir-backmap.height+2)
			engine.draw(tiledmap.surface, tiledmap.width/2, tiledmap.height/2)
			if tiledmap == elevator_map:
				nfilled = str(demo.floor)
				spaces = ' '*(4-len(nfilled))
				engine.draw(dfont.render(spaces+nfilled, 1, (0,155,0)), 3.55,2.5)

			for p in particles:
				p.update(engine.delta)
				engine.draw_pixel(p.pos.x, p.pos.y, p.get_color())
				if p.is_finished():
					particles.remove(p)

			if tiledmap == elevator_map:
				if elevator_timer > 0.0:
					quake += engine.delta * 0.5 * (1.0+(elevator_speed-1.0)*0.5)
				camera.pos.x = 4.0+(random.random()-0.5)*quake
				camera.pos.y = 2.5+(random.random()-0.5)*quake
			else:
				camera.pos.x = player.pos.x
				camera.pos.y = player.pos.y

			for person in persons:
				pos = person.pos
				size = person.size
				half = person.half
				anim = 'idle' if tiledmap.lighted or not 'idle_dark' in person.animations else 'idle_dark'
				if person.input['left'] or person.input['right']:
					anim = 'walk' if tiledmap.lighted else 'walk_dark'
					if not anim in person.animations:
						anim = 'walk'
				if person == player and interaction_timer > 0.0:
					if flashlight:
						anim = 'interaction_dark'
					else:
						anim = 'interaction'
				engine.draw(person.animations[anim].get(engine.timer, person.dir < 0), pos.x, pos.y)
			engine.window.blit(engine.lighting, (0,0), special_flags=pygame.BLEND_RGBA_SUB)

			if player.dead and deadtimer < 0.8 and engine.frameid % 2 == 0 and screamer:
				engine.window.blit(bleedscreen, (0,0))
			engine.window.blit(vignette, (0,0))

			if player.dead and deadtimer > 1.0:
				engine.window.blit(font.render("you are dead", 1, (int(min(100, 100*(deadtimer-1))),0,0)), (engine.width/2-100,engine.height/2-16))

			if flashlight:
				e = math.ceil(energy*5)
				for i in range(5):
					color = (0,100,0)
					if e <= 3:
						color = (100,100,0)
					if e <= 1:
						color = (100,0,0)
					if e <= i:
						color = (color[0]//2, color[1]//2, color[2]//2)

					pygame.draw.rect(engine.window, color, (10,12*(5-i+1), 50,10))
			# --- draw inventory content
			y = 0
			for itemname in inventory:
				count = inventory[itemname]
				if count:
					engine.window.blit(items_textures[itemname], (engine.width-engine.UNIT,y))
					engine.window.blit(dfont.render(str(count), 1, (255,255,255)), (engine.width-engine.UNIT//2+5, y+20))
					y += engine.UNIT
			# ---

			if battle:
				pygame.draw.rect(engine.window, (100,100,100), (engine.width//2-100, engine.height-100, 200, 10))
				pygame.draw.rect(engine.window, (255,255,255), (engine.width//2-100, engine.height-100, int(200*battle_value), 10))
				if (engine.frameid//2) % 2 == 0:
					engine.window.blit(attackfont.render('TAP SPACE AS FAST AS YOU CAN', 1, (255,100,100)), (engine.width//2-160, engine.height-80))
				else:
					engine.window.blit(attackfont.render('TAP SPACE AS FAST AS YOU CAN', 1, (0,0,100)), (engine.width//2-1260, engine.height-80))

			if note_on and not player.dead:
				engine.window.blit(note_img, (100,50))
				engine.window.blit(notefont.render("RECOVER HELP:   from 10/12/2000", 1, (7,5,10)), (110, 140))
				engine.window.blit(notefont.render("- when you hear noises", 1, (7,5,10)), (110, 160))
				engine.window.blit(notefont.render("similar to breath - don't move", 1, (7,5,10)), (110, 170))
				engine.window.blit(notefont.render("- find fuel on floors", 1, (7,5,10)), (110, 200))
				engine.window.blit(notefont.render("- be careful with elevator", 1, (7,5,10)), (110, 210))
				engine.window.blit(notefont.render("- your task is - start generators on", 1, (7,5,10)), (110, 220))
				engine.window.blit(notefont.render("generators are on floors - 0, 10, 20", 1, (7,5,10)), (110, 230))
				engine.window.blit(notefont.render("- find generators in small 2x2 rooms", 1, (7,5,10)), (110, 240))
				engine.window.blit(notefont.render("- floors -1, 21,22,27 are out of order", 1, (7,5,10)), (110, 250))
				engine.window.blit(notefont.render("may be reused until 01/01/2002", 1, (7,5,10)), (110, 360))
				engine.window.blit(handfont.render("D.Noviy lost 29/12/2001", 1, (50,50,50)), (110, 386))
				engine.window.blit(handfont.render("K.Wildhill lost 10/01/2001", 1, (50,50,50)), (110, 398))
				engine.window.blit(handfont.render("L.De lost 27/02/2001", 1, (50,50,50)), (150, 410))

				engine.window.blit(notefont.render("press 'H' to hide help-list", 1, (255,255,255)), (110, 470))
			#engine.window.blit(notefont.render("similar to breath - don't move", 1, (7,5,10)), (110, 170))

			if fadetimer > 0.0:
				fadetimer -= engine.delta*0.5
				if fadetimer < 0.0:
					if not fadein:
						fadetimer = 1.0
						fadein = True
					else:
						fadetimer = 0.0
				if fadein:
					pygame.gfxdraw.box(engine.window, (0,0,engine.width, engine.height), (0,0,0,int(fadetimer*255)))
				else:
					pygame.gfxdraw.box(engine.window, (0,0,engine.width, engine.height), (0,0,0,255-int(fadetimer*255)))

		if paused:
			engine.window.fill((0,0,0))
			engine.window.blit(notefont.render("<Game paused> press Esc to continue or Q for quit", 1, (255,255,255)), (50, 50))
		engine.show_fps()
		if player.dead and deadtimer < 0.8:
			engine.flip(20)
		else:
			engine.flip()
