import random
from .maths import *
from . import engine
from . import sound
from . import demo

def AI_player(self, tiledmap, persons, delta):
	pass

def AI_knocker(self, tiledmap, persons, delta):
	pass

def AI_thing(self, tiledmap, persons, delta):
	if self.state == 'escape' and self.victim.pos.dst2(self.pos) > 13**2:
		self.pos.x += 100
		self.input['left'] = False
		self.input['right'] = False
		return

	if self.state == 'attack' and self.victim.pos.dst2(self.pos) < 2**2:
		self.victim.dead = True
		return

	if self.state == 'hide':
		if self.dir > 0 and self.pos.x+1 >= self.cabinet:
			sound.play('cabinet')
			self.pos.x += 1e8
			self.state = 'idle'
			tiledmap.set(self.cabinet, self.cabinet_y, 'furniture', 66, False)
			tiledmap.set(self.cabinet, self.cabinet_y-1, 'furniture', 67, False)

		if self.dir < 0 and self.pos.x-1 <= self.cabinet:
			sound.play('cabinet')
			self.pos.x += 1e8
			self.state = 'idle'

	if self.attack_timer > 0.0:
		self.attack_timer -= delta
		self.agression += self.victim.is_moving()*delta
		if ((self.lighted != self.victim.flashlight and self.lighted) or self.agression > 0.75) and self.state != 'attack':
			self.agression += 0.75
			self.state = 'attack'
			sound.play('many_steps')
			if self.victim.pos.x > self.pos.x:
				self.input['right'] = True
			else:
				self.input['left'] = True
		elif self.attack_timer <= 0.0:
			if tiledmap.width < 18:
				self.state = 'hide'
				self.attack_timer = 0.0
				sound.play('many_steps_out')
				cabinet = random.choice(tiledmap.objects['cabinets'])
				self.cabinet, self.cabinet_y = cabinet[0:2]
				if self.pos.x < self.cabinet:
					self.input['right'] = True
				else:
					self.input['left'] = True
			else:
				self.state = 'escape'
				self.attack_timer = 0.0	
				sound.play('many_steps_out')
				if self.victim.pos.x > self.pos.x:
					self.input['left'] = True
					self.dir = -1
				else:
					self.input['right'] = True
					self.dir = 1
				demo.things.remove(demo.floor)

	elif self.state == 'none':
		for person in persons:
			if person.ai == AI_PLAYER and person.pos.dst2(self.pos) < 11**2:
				self.attack_timer = 4.20
				self.agression = 0.0
				self.victim = person
				self.lighted = person.flashlight
				sound.play('thing_breath')
				self.state = 'wait'
				break

AI_PLAYER = 'player'
AI_THING1 = 'thing-1'
AI_KNOCKER = 'knocker'
AIs = {
	'player':AI_player,
	'knocker':AI_knocker,
	'thing-1': AI_thing,
}

class Person:
	def __init__(self, pos, size, speed, animations, ai):
		self.pos = pos
		self.size = size
		self.half = vec2(size.x*0.5, size.y*0.5)
		self.vel = vec2(0.0,0.0)
		self.dir = 1
		self.speed = speed
		self.animations = animations
		self.input = {'left':False, 'right':False}
		self.ai = ai

		self.attack_timer = 0.0
		self.agression = 0.0
		self.flashlight = False
		self.state = 'none'
		self.stepstimer = 0.0
		self.dead = False

	def is_moving(self):
		return self.input['left'] or self.input['right']

	def update_AI(self, tiledmap, persons, delta):
		ai = AIs[self.ai]
		ai(self, tiledmap, persons, delta)

	def update_physics(self, tiledmap, delta, steps=10):
		pos = self.pos
		vel = self.vel
		size = self.size
		half = self.half

		self.onground = False

		lastx = pos.x
		lasty = pos.y

		for i in range(steps):
			pos.y += vel.y*delta/steps
			pos.x += vel.x*delta/steps
			vel.y -= engine.delta*9.8*2/steps

			E = 0.05

			rightwall = tiledmap.get_tilecol(int(pos.x+half.x), int(pos.y-half.y+E)) or \
						tiledmap.get_tilecol(int(pos.x+half.x), int(pos.y+half.y-E))

			if rightwall:
				pos.x = int(lastx)+1-half.x-0.001
				vel.x = 0.0

			leftwall = tiledmap.get_tilecol(int(pos.x-half.x), int(pos.y-half.y+E)) or \
					   tiledmap.get_tilecol(int(pos.x-half.x), int(pos.y+half.y-E))

			if leftwall:
				pos.x = int(lastx)+half.x+0.001
				vel.x = 0.0

			onground = tiledmap.get_tilecol(int(pos.x-half.x+E), int(pos.y-half.y)) or \
					   tiledmap.get_tilecol(int(pos.x+half.x-E), int(pos.y-half.y))
			if onground:
				pos.y = int(pos.y)+half.y
				vel.y = 0.0
				self.onground = True

			ceiling = tiledmap.get_tilecol(int(pos.x-half.x+E), int(pos.y+half.y)) or \
					  tiledmap.get_tilecol(int(pos.x+half.x-E), int(pos.y+half.y))
			if ceiling:
				pos.y = int(pos.y+half.y)-half.y
				if vel.y > 0.0:
					vel.y = 0.0

		if onground:
			vel.x -= vel.x*min(1.0, delta*15)
		else:
			vel.x -= vel.x*min(1.0, delta*2.5)


	def controls(self, tiledmap, delta):
		pos = self.pos
		half = self.half

		left = self.input['left']
		right = self.input['right']

		if left:
			self.dir = -1
		if right:
			self.dir = 1

		if (left or right) and self.ai == AI_PLAYER:
			self.stepstimer += delta
			if self.stepstimer > 0.4:
				self.stepstimer %= 0.4
				sound.play('%sstep_%s' % (tiledmap.stepprefix, random.randint(0,3)))

		speed = self.speed
		if left and self.vel.x > -speed:
			self.vel.x -= min(1.0, delta*15)*speed

		if right and self.vel.x < speed:
			self.vel.x += min(1.0, delta*15)*speed
